package org.devcake.groovy.autoimports

import org.codehaus.groovy.ast.ImportNode

import org.codehaus.groovy.ast.PackageNode
import org.codehaus.groovy.control.CompilerConfiguration
import org.codehaus.groovy.control.CompilationFailedException
import org.codehaus.groovy.control.CompilationUnit
import java.security.CodeSource
import org.codehaus.groovy.control.Phases
import org.codehaus.groovy.control.CompilationUnit.SourceUnitOperation
import org.codehaus.groovy.control.SourceUnit

public static class AutoImportsGroovyClassLoader extends IntermediateClassLoader {
    List<ImportNode> defaultImports = []
    List<ImportNode> starImports = []
    Map<String, ImportNode> staticStarImports = [:]
    Map<String, ImportNode> staticImports = [:]
    PackageNode defaultPackage
    String targetFileName
    private boolean applyDefaultImports = false

    public AutoImportsGroovyClassLoader() {
        super()
    }
    public AutoImportsGroovyClassLoader(ClassLoader loader, CompilerConfiguration config) {
        super(loader, config)
    }
    public Class parseClass(GroovyCodeSource codeSource, boolean shouldCacheSource) throws CompilationFailedException {
        if (codeSource.getName() == targetFileName) {
            applyDefaultImports = true
        }
        try {
            return parseClassSuper(codeSource, shouldCacheSource)
        } finally {
            applyDefaultImports = false
        }
    }

    protected CompilationUnit createCompilationUnit(CompilerConfiguration config, CodeSource source) {
        CompilationUnit cu = createCompilationUnitSuper(config, source)
        if (applyDefaultImports) {
            def importsOperation = new AutoImportsOperation(
                loader:this,
                defaultImports:defaultImports,
                starImports:starImports,
                staticStarImports:staticStarImports,
                staticImports:staticImports)

            cu.addPhaseOperation(importsOperation, Phases.CONVERSION)
            cu.addPhaseOperation(new SourceUnitOperation() {
                void call(SourceUnit sourceUnit) {
                    if (defaultPackage?.name) {
                        if (!sourceUnit.getAST().packageName) {
                            sourceUnit.getAST().package = defaultPackage
                        }
                        sourceUnit.getAST().classes.each {
                            if (!it.packageName) {
                                it.name = defaultPackage.name + it.name
                            }
                        }
                    }
                }
            }, Phases.CONVERSION)
        }
        return cu
    }

    public boolean classIsAvailable(String className) {
        def end = false
        /* we are looking for specified class, and also check, maybe it is internal class
         * like java.util.Map$Entry or java.util$Map$Entry or ...
         */
        while(!end) {
            end = !className.contains('.')
            try {
                loadClass(className, false);
                return true
            } catch (ClassNoFoundException) {
            }
            className = className.replaceAll(/^(.*)\.([^\.]*)$/, /$1\$$2/)
        }
        return false
    }
}
