package org.devcake.groovy.autoimports

/**
 * Created by IntelliJ IDEA.
 * User: Admin
 * Date: 12.08.2010
 * Time: 9:43:11
 * To change this template use File | Settings | File Templates.
 */

import org.codehaus.groovy.control.SourceUnit
import org.codehaus.groovy.control.CompilationUnit
import org.codehaus.groovy.control.CompilerConfiguration
import org.codehaus.groovy.control.Phases
import org.codehaus.groovy.control.CompilationFailedException
import org.codehaus.groovy.ast.ImportNode
import org.codehaus.groovy.ast.ModuleNode

/**
 * Helping class that allows to run selected part of script in context of whole script.
 *
 * @author Sergii Bondarenko
 *
 * Created on 2009-02-01
 */
public class AutoImportsGroovyShell extends GroovyShell {
    public AutoImportsGroovyShell() {
        this(null, new Binding())
    }

    public AutoImportsGroovyShell(Binding binding) {
        this(null, binding)
    }

    public AutoImportsGroovyShell(CompilerConfiguration config) {
        this(new Binding(), config)
    }

    public AutoImportsGroovyShell(Binding binding, CompilerConfiguration config) {
        this(null, binding, config)
    }

    public AutoImportsGroovyShell(ClassLoader parent, Binding binding) {
        this(parent, binding, CompilerConfiguration.DEFAULT)
    }

    public AutoImportsGroovyShell(ClassLoader parent) {
        this(parent, new Binding(), CompilerConfiguration.DEFAULT)
    }
    public AutoImportsGroovyShell(ClassLoader parent, Binding binding, final CompilerConfiguration config) {
        super(parent, binding, config)
        ClassLoader parentLoader = (parent!=null)?parent:GroovyShell.class.getClassLoader()
        GroovyShell.class.metaClass.setAttribute(this, 'loader', new AutoImportsGroovyClassLoader(parentLoader,config))
    }

    public Object runSelected(String fullScriptText, String selectedText, String fileName, List list) throws CompilationFailedException {
        def classLoader = (AutoImportsGroovyClassLoader) getClassLoader()
        try {
            try {
                def fullSource = new GroovyCodeSource(fullScriptText, 'FullScript', '/groovy.shell')
                def loader = new GroovyClassLoader()
                def compilationUnit = new CompilationUnit(CompilerConfiguration.DEFAULT, fullSource.getCodeSource(), loader)
                def su = compilationUnit.addSource('FullScript', new ByteArrayInputStream(fullScriptText.getBytes("UTF-8")))
                compilationUnit.compile(Phases.CONVERSION)
                classLoader.defaultImports = su.AST.imports
                classLoader.starImports = su.AST.starImports
                classLoader.staticStarImports = su.AST.staticStarImports
                classLoader.staticImports = su.AST.staticImports
                classLoader.defaultPackage = su.AST.package
                classLoader.targetFileName = fileName;
            } catch (CompilationFailedException e) {
                //e.printStackTrace(System.out)
                //since we are trying to run selected part of script, compilation errors in whole script are silently ignored
            }
            return run(selectedText, fileName, list as String[])
        } catch(CompilationFailedException e) {
            //e.printStackTrace(System.out)
            throw e
        } finally {
            classLoader.defaultImports = []
            classLoader.starImports = []
            classLoader.staticStarImports = [:]
            classLoader.staticImports = [:]
            classLoader.targetFileName = null;
        }
    }

}

private static class AutoImportsOperation extends CompilationUnit.SourceUnitOperation {
    AutoImportsGroovyClassLoader loader
    List<ImportNode> defaultImports = []
    List<ImportNode> starImports = []
    Map<String, ImportNode> staticStarImports = [:]
    Map<String, ImportNode> staticImports = [:]

    public void call(SourceUnit source) throws CompilationFailedException {
        ModuleNode module = source.getAST()
        defaultImports.each{ if (module.getImport(it.alias) == null && loader.classIsAvailable(it.type.name)) module.addImport(it.alias, it.type) }
        starImports.each{ module.addStarImport(it.packageName) }
        staticStarImports.each{ if (loader.classIsAvailable(it.value.type.name)) module.addStaticStarImport(it.key, it.value.type) }
        staticImports.each{ if (!module.staticImports.containsKey(it.key) && loader.classIsAvailable(it.value.type.name)) module.addStaticImport(it.value.type, it.key, it.key) }
    }
}