package org.devcake.groovy.autoimports;

import groovy.lang.GroovyClassLoader;
import groovy.lang.GroovyCodeSource;
import org.codehaus.groovy.control.CompilationUnit;
import org.codehaus.groovy.control.CompilerConfiguration;

import java.security.CodeSource;

/**
 * The purpose of this class is to avoid StackOverflowError (that occurs only in Google App Engine).
 *
 * Example of stecktrace we are trying to avoid:
 *
 * java.lang.StackOverflowError
 * at java.security.AccessController.doPrivileged(Native Method)
 * at java.io.PrintWriter.(PrintWriter.java:95)
 * at java.io.PrintWriter.(PrintWriter.java:79)
 * at java.util.logging.SimpleFormatter.format(SimpleFormatter.java:93)
 * at java.util.logging.StreamHandler.publish(StreamHandler.java:196)
 * at java.util.logging.ConsoleHandler.publish(ConsoleHandler.java:105)
 * at java.util.logging.Logger.log(Logger.java:474)
 * at java.util.logging.Logger.doLog(Logger.java:495)
 * at java.util.logging.Logger.log(Logger.java:584)
 * at java.lang.reflect.Method.invoke(Method.java:48)
 * at org.codehaus.groovy.reflection.CachedMethod.invoke(CachedMethod.java:88)
 * at groovy.lang.MetaMethod.doMethodInvoke(MetaMethod.java:233)
 * at groovy.lang.MetaClassImpl.invokeMethod(MetaClassImpl.java:1058)
 * at groovy.lang.ExpandoMetaClass.invokeMethod(ExpandoMetaClass.java:1070)
 * at org.codehaus.groovy.runtime.ScriptBytecodeAdapter.invokeMethodOnSuperN(ScriptBytecodeAdapter.java:127)
 * at org.devcake.groovy.autoimports.AutoImportsGroovyClassLoader.parseClass(AutoImportsGroovyShell.groovy:114)
 * at sun.reflect.GeneratedMethodAccessor51.invoke(Unknown Source)
 * at sun.reflect.DelegatingMethodAccessorImpl.invoke(DelegatingMethodAccessorImpl.java:43)
 * at java.lang.reflect.Method.invoke(Method.java:43)
 * at org.codehaus.groovy.reflection.CachedMethod.invoke(CachedMethod.java:88)
 * at groovy.lang.MetaMethod.doMethodInvoke(MetaMethod.java:233)
 * at groovy.lang.MetaClassImpl.invokeMethod(MetaClassImpl.java:1058)
 * at groovy.lang.ExpandoMetaClass.invokeMethod(ExpandoMetaClass.java:1070)
 *
 * User: Sergey Bondarenko
 * Date: 2/7/11
 * Time: 6:00 AM
 */
public class IntermediateClassLoader extends GroovyClassLoader {
    public IntermediateClassLoader(ClassLoader loader, CompilerConfiguration config) {
        super(loader, config);
    }

    public Class parseClassSuper(GroovyCodeSource codeSource, boolean shouldCacheSource) {
        return super.parseClass(codeSource, shouldCacheSource);
    }

    protected CompilationUnit createCompilationUnitSuper(CompilerConfiguration config, CodeSource source) {
        return super.createCompilationUnit(config, source);
    }

}
